local addonInfo, privateVars = ...

if nkPanelPatron ~= nil then return end
nkPanelPatron = {}

function nkPanelPatron.main(_, addon)

	if addon == addonInfo.identifier then
	
		nkPanel.api.registerPlugin('Patron plugin', nkPanelPatron.plugin())
	end
	
end

Command.Event.Attach(Event.Addon.Load.End, nkPanelPatron.main, "nkPanelPatron.Addon.Load.End")

--------------------------------------------------------------------------------------------------------------------------------------------

function nkPanelPatron.plugin()
		
	local plugin = nkPanel.api.plugin('Patron plugin', true, false)

	plugin:SetTitle(nkPanelPatron.langTexts.title)
	plugin:SetSecureMode('restricted')
	
	plugin:SetThumb('nkPanel_Patron', 'gfx/nkPanelIcon.png')
	
	local function playerReady (_, units)
	
		if playerUNID == nil then
			local details = Inspect.Unit.Detail('player')
			if details ~= nil then playerUNID = details.id end
			if playerUNID == nil then return end
		end
		
		if units[playerUNID] == nil then return end

		local body = plugin:CreateBody()
		
		body:SetSecureMode('restricted')
		
		local menu = EnKai.uiCreateFrame("nkMenu", 'nkPanelPatron.panelMenu', body)
			
		menu:SetFontSize(nkPanelSetup.fontSize)
		menu:SetWidth(120)
		menu:SetBackgroundColor(0, 0, 0, 0)
		menu:SetLabelColor(1, 1, 1, 1)
		menu:SetBorderColor(0, 0, 0, 0)
		menu:SetPoint("TOPLEFT", body, "TOPLEFT", 2, 2)
		
		local items = {}
		local subMenus
		
		function plugin:AddSubMenu (label, subMenuItems, mainFunc)
			
			local newSubMenu = EnKai.uiCreateFrame("nkMenu", 'nkPanelPatron.panelMenu.' .. label, body)
			newSubMenu:SetFontSize(nkPanelSetup.fontSize)
			newSubMenu:SetWidth(120)
			newSubMenu:SetBackgroundColor(0, 0, 0, .8)
			newSubMenu:SetLabelColor(1, 1, 1, 1)
			newSubMenu:SetBorderColor(0.925, 0.894, 0.741, 1)
			newSubMenu:SetVisible(false)
			newSubMenu:SetID(label)
			
			local showSubMenu = function ()
				local state = newSubMenu:GetVisible()
			
				for k, v in pairs (subMenus) do v:SetVisible(false) end
				
				if state == true then 
					 if nkPanel.api.setBodyClose ~= nil then -- check for nkPanel V1.1.1 change 
					 	nkPanel.api.setBodyClose (body)
					 end					 
				elseif newSubMenu:GetEntryCount() > 0 then
					newSubMenu:SetVisible(true)
					if nkPanel.api.preventBodyClose ~= nil then -- check for nkPanel V1.1.1 change 
						nkPanel.api.preventBodyClose ()
						plugin:SetUpdate(function() nkPanel.api.setBodyClose (body) end, true, 10)						
					end
				end 
			end
			
			for k, v in pairs(subMenuItems) do
				local thisEntry
							
				if v.seperator == true then
					thisEntry = newSubMenu:AddSeparator()					
				elseif v.callBack ~= nil then
					thisEntry = newSubMenu:AddEntry({ closeOnClick = true, label = v.label, macro = v.macro, callBack = function() v.callBack() end })
					thisEntry:SetID(v.id)
				else
					thisEntry = newSubMenu:AddEntry({ closeOnClick = true, label = v.label, macro = v.macro })
					thisEntry:SetID(v.id)
				end
				
				if v.CD ~= nil then
					thisEntry:SetBackgroundColor(0.5, 0, 0.216, 1)
				end			
				
			end
						
			local mainMenuItems = menu:GetEntries()

			if #mainMenuItems > 0 then
				newSubMenu:SetPoint("TOPRIGHT", mainMenuItems[#mainMenuItems], "TOPLEFT", -3, 18)
			else
				newSubMenu:SetPoint("TOPRIGHT", menu, "TOPLEFT", -2, -1)
			end
			
			if subMenus == nil then subMenus = {} end
			table.insert(subMenus, newSubMenu)
			
			menu:AddEntry ( { subMenu = true, label = nkPanelPatron.langTexts[label], callBack = function () showSubMenu() end } )
		
		end
			
			
		local abilityList = {
			utility = {
				"A3D21C84687199B44", -- "Bankier beschwören"
				"A2EA96AD566F13C62", --"Gilden-Bankier beschwören"
				"A3F5DD24F7FEF4C53", -- "Mailbox des Patronen"
				"A0AE89C20B7D46D0F", -- "Privat-Mailbox beschwören"
				"A3416A1A673AEA8DF" -- "Veteranenhändler beschwören"
			},
			
			teacher = {
				"AFD83E703841348C0", -- "Magier-Ausbilder beschwören"
				"A72F1082A8F6DD4C5", -- "Krieger-Ausbilder beschwören"
				"A4090FAAA693ABEEC", -- "Schurken-Ausbilder beschwören"
				"A439CC66597456366" -- "Kleriker-Ausbilder beschwören"
			},
			
			buffs = {
				"A0530A5F074940F78", -- "Artefaktaufspürer des Stammspielers"
				"A3F5E814D94343B3C", -- "Bonus-Dungeonladungen"
				"A54F1DA73C9D1D272", -- "Bonus-Kriegsfrontladungen"
				"A3C7F03BBA1BBE8AD", -- "Erfahrungsbonus-Unterdrückung"
				"A665FDAC7EDD37636", -- "Passierschein des Stammspielers"				
			}		
		}
	
		local playerAbilityList = Inspect.Ability.New.List()
		
		for category, abilities in pairs (abilityList) do
			local menuItems = {}
		
			for idx = 1, # abilities, 1 do
				if playerAbilityList[abilities[idx]] ~= nil then
					local details = Inspect.Ability.New.Detail(abilities[idx])
					if details ~= nil then
						table.insert(menuItems, { id = abilities[idx], label = details.name, macro = string.format('cast %s', details.name), CD = details.currentCooldownRemaining })
					end
				end
			end
			
			if #menuItems > 0 then plugin:AddSubMenu (category, menuItems, nil) end
		end
			
		body:SetHeight(menu:GetHeight()+3)
		body:SetWidth(menu:GetWidth()+3)
		
		local function processCooldown (cooldowns, active)
		
			local flag = true
	
			for k, v in pairs (cooldowns) do
			
				if v > 1.5 then
			
					for category, list in pairs(abilityList) do				
						for idx = 1, #list, 1 do
							if list[idx] == k then						
								for idx2 = 1, #subMenus, 1 do
									if subMenus[idx2]:GetID() == category then
										local entry = subMenus[idx2]:GetEntryByID(k)
										
										if active == true then
											entry:SetBackgroundColor(0.5, 0, 0.216, 1)
										else
											entry:SetBackgroundColor(0,0,0,1)
										end
										
										flag = false
									end
								end
							end
							if flag == false then break end
						end
						if flag == false then break end
					end
					if flag == false then break end
				end
			end
		
		end
		
		local function cooldownEnd (_, cooldowns) processCooldown (cooldowns, false) end 
		local function cooldownBegin (_, cooldowns) processCooldown (cooldowns, true) end
		
		Command.Event.Attach(Event.Ability.New.Cooldown.Begin, cooldownBegin, "nkPanelPatron.Cooldown.Begin")
		Command.Event.Attach(Event.Ability.New.Cooldown.End, cooldownEnd, "nkPanelPatron.Cooldown.End")

		Command.Event.Detach(Event.Unit.Availability.Full, nil, "nkPanelPatron.Unit.Availability.Full")
	end
	
	Command.Event.Attach(Event.Unit.Availability.Full, playerReady, "nkPanelPatron.Unit.Availability.Full")
	
	return plugin

end